SystemOrganization addCategory: #'CodeMine-Text'!
SystemOrganization addCategory: #CodeMine!

Object subclass: #CMCallGraph
	instanceVariableNames: 'selectorsMap methods'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 20:32'!
fromCategory: aString
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString.
	^callGraph! !

!CMCallGraph class methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:09'!
fromCategory: aString excluding: aBlock
	"Compute call graph from categories beginning with aString"

	| callGraph |
	
	callGraph := CMCallGraph new.
	(CMCallGraphBuilder forGraph: callGraph) scanCategoryPrefix: aString excluding: aBlock.
	^callGraph! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:54'!
addMethod: aCMMethod
	methods add: aCMMethod
	
	! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:07'!
initialize 
	super initialize.
	selectorsMap := Dictionary new.
	methods := OrderedCollection new.! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods
	^ methods! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
methods: anObject
	methods := anObject! !

!CMCallGraph methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
selectorFor: aSymbol
	
	^selectorsMap
		at: aSymbol 
		ifAbsentPut: [ CMSelector named: aSymbol ]! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap
	^ selectorsMap! !

!CMCallGraph methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selectorsMap: anObject
	selectorsMap := anObject! !

Object subclass: #CMCallingRelation
	instanceVariableNames: 'previous next container selector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container
	^ container! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
container: anObject
	container := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
next
	^ next! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
next: anObject
	next := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
previous
	^ previous! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
previous: anObject
	previous := anObject! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector
	^ selector! !

!CMCallingRelation methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
selector: anObject
	selector := anObject! !

Object subclass: #CMCodeTopicModel
	instanceVariableNames: 'methodTexts featureSpace topicModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:38'!
computeLdaModel: iterations
	
	| lda |

	lda := LDAllocator new.
		
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:41'!
computeTopics: numTopics forIterations: iterations
	
	| lda |

	lda := LDAllocator new.
	self topicModel: (lda
		allocate: (self methodTexts values) 
		iterating: iterations 
		features: (self featureSpace featureCount) 
		topics: numTopics).
	^self topicModel
	
	! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 23:43'!
extractTextFromCallGraph: aCallGraph
	"Collect all textual tokens from a call graph, eliminate non-frequent tokens
	and construct the feature space from the rest"

	| frequencies methodFeatures |
	frequencies := Dictionary new.
	methodFeatures := OrderedDictionary new.
	
	aCallGraph methods do: [ :method | | textVisitor |
		textVisitor := CMTextVisitor new.
		textVisitor visitNode: (method sourceTree).
		textVisitor literals do: [ :symbol |
			frequencies at: symbol put: (1 + (frequencies at: symbol ifAbsentPut: 0)).
		].
		methodFeatures at: method put: (textVisitor literals).
	].
	methodFeatures keysAndValuesDo: [ :method :features | | reducedFeatures |
		reducedFeatures := features select: [:feature | (frequencies at: feature) > 2 ].
		self methodTexts
			at: method 
			put: (self featureSpace representAll: reducedFeatures).
		]! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace
	^ featureSpace! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
featureSpace: anObject
	featureSpace := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:35'!
initialize 
	
	methodTexts := OrderedDictionary new.
	featureSpace := LDFeatureSpace new.! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts
	^ methodTexts! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:30'!
methodTexts: anObject
	methodTexts := anObject! !

!CMCodeTopicModel methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:53'!
sortedTopic: topic
	
	^(((self topicModel featuresForTopicAt: topic) 
		withIndexCollect: [ :value :index | 
			Array with: value with: index]) 
		sorted: [ :a :b | (a first) > (b first) ])
		collect: [ :pair | self featureSpace interpret: (pair second) ]
	! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel
	^ topicModel! !

!CMCodeTopicModel methodsFor: 'accessing' stamp: 'toma 8/12/2015 22:34'!
topicModel: anObject
	topicModel := anObject! !

Object subclass: #CMMethod
	instanceVariableNames: 'implementedSelector callingRelations lastAdded sourceTree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations
	^ callingRelations! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
callingRelations: anObject
	callingRelations := anObject! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector
	^ implementedSelector! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementedSelector: anObject
	implementedSelector := anObject! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:35'!
initialize 
	super initialize.
	callingRelations := OrderedCollection new.! !

!CMMethod methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 18:31'!
newCallTo: aSelector 

	| call |
	call := CMCallingRelation new.
	call
		previous: (lastAdded);
		selector: aSelector;
		container: self.
	lastAdded ifNotNilDo: [ :this | this next: call].
	lastAdded := call.
	aSelector senders add: call.
	self callingRelations add: call.
	^call
	! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 20:49'!
sourceTree
	^sourceTree! !

!CMMethod methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:08'!
sourceTree: aSourceAst
	sourceTree := aSourceAst! !

Object subclass: #CMSelector
	instanceVariableNames: 'symbol implementors senders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMSelector class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:14'!
named: aSymbol

	^(CMSelector new)
		symbol: aSymbol;
		yourself! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors
	^ implementors! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
implementors: anObject
	implementors := anObject! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 17:13'!
initialize
	implementors := OrderedCollection new.
	senders := OrderedCollection new.! !

!CMSelector methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:08'!
newImplementor: aSourceAst
	
	| method |

	method := CMMethod new.
	method 
		implementedSelector: self;
		sourceTree: aSourceAst.
	self implementors add: method.
	^method
! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders
	^ senders! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
senders: anObject
	senders := anObject! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol
	^ symbol! !

!CMSelector methodsFor: 'accessing' stamp: 'toma 8/12/2015 17:08'!
symbol: anObject
	symbol := anObject! !

RBProgramNodeVisitor subclass: #CMCallGraphBuilder
	instanceVariableNames: 'currentMethod callGraph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine'!

!CMCallGraphBuilder class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 18:02'!
forGraph: aCallGraph
	
	^(CMCallGraphBuilder new)
		callGraph: aCallGraph;
		yourself.
		! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 18:45'!
acceptMessageNode: aMessageNode
	| selector |

	super acceptMessageNode: aMessageNode.
	selector := self callGraph selectorFor: aMessageNode selector.
	self currentMethod newCallTo: selector.

	! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:09'!
acceptMethodNode: aMethodNode
	
	| selector |
	selector := self callGraph selectorFor: (
		aMethodNode selector asSymbol).
	self currentMethod: (selector newImplementor: aMethodNode).
	self callGraph methods add: (self currentMethod).
	super acceptMethodNode: aMethodNode.! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph
	^ callGraph! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
callGraph: anObject
	callGraph := anObject! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod
	^ currentMethod! !

!CMCallGraphBuilder methodsFor: 'accessing' stamp: 'toma 8/12/2015 18:02'!
currentMethod: anObject
	currentMethod := anObject! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/14/2015 11:57'!
scanCategoryPrefix: aString

	^self scanCategoryPrefix: aString excluding: [ :class | false ]
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:33'!
scanCategoryPrefix: aString excluding: aBlock

	((Object allSubclasses
		select: [ :class |
			class category ifNil: [false] ifNotNilDo: [:cat | cat beginsWith: aString]])
		select: [ :class |
			(aBlock value: class) not]) 
		do: [: class |
			self scanClass: class].
! !

!CMCallGraphBuilder methodsFor: 'as yet unclassified' stamp: 'toma 8/13/2015 15:37'!
scanClass: aClass

	aClass methodDict valuesDo: [ :method | 	| node |
		node := RBParser parseMethod: (method getSource asString).
		self visitNode: node.
	]
! !

RBProgramNodeVisitor subclass: #CMTextVisitor
	instanceVariableNames: 'literals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CodeMine-Text'!

!CMTextVisitor class methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 23:18'!
normalize: aString
	"Splits a camel case selector into lowercase atoms without colons"
	
	| tokens lastIndex string |

	string := aString.
	tokens := OrderedCollection new.
	lastIndex := 1.
	string withIndexDo: [ :char :index |
		(char isUppercase or: [char isLetter not]) ifTrue: [
			(index > 1) ifTrue: [
				tokens add: (string copyFrom: lastIndex to: (index - 1)) asLowercase.
			].
			lastIndex := index.
		]
	].
	(lastIndex <= (string size)) ifTrue: [
		tokens add: (string copyFrom: lastIndex to: (string size)) asLowercase.
	].
	^(tokens collect: [:token | token onlyLetters]) select: [:token | token size >= 3]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptLiteralNode: aLiteralNode
	| value |
	super acceptLiteralNode: aLiteralNode.
	value := aLiteralNode token value.
	(value isString)
		ifTrue: [(CMTextVisitor normalize: value) do: [:atom | self literals add: atom]]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:24'!
acceptMessageNode: aMessageNode 
	
	super acceptMessageNode: aMessageNode.
	(CMTextVisitor normalize: (aMessageNode selector)) do: [ :atom |
		self literals add: atom]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptMethodNode: aMethodNode
	
	super acceptMethodNode: aMethodNode.
	(CMTextVisitor normalize: (aMethodNode selector)) do: [ :atom |
		self literals add: atom]! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 22:26'!
acceptVariableNode: aVariableNode

	super acceptVariableNode: aVariableNode.
	(CMTextVisitor normalize: (aVariableNode name)) do: [ :atom |
		self literals add: atom]
	! !

!CMTextVisitor methodsFor: 'as yet unclassified' stamp: 'toma 8/12/2015 21:36'!
initialize 

	super initialize.
	literals := OrderedCollection new.! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals
	^ literals! !

!CMTextVisitor methodsFor: 'accessing' stamp: 'toma 8/12/2015 21:38'!
literals: anObject
	literals := anObject! !
